<?xml version="1.0" encoding="UTF-8"?>
<!-- stylesheet for references validation v1.0 -->
<xsl:stylesheet version="1.0" xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
                xmlns:source="http://lca.jrc.it/ILCD/Source"
                xmlns:common="http://lca.jrc.it/ILCD/Common">

    <!-- Changes:

    1.2
        - remove unused NS declarations
        - import common-1.2

    1.1
        - check file names of external files for allowed characters are a-z A-Z 0-9 . - _ , ( ) [space]

    -->

    <xsl:import href="common-1.2.xsl"/>

    <xsl:variable name="referenceTypes" select="document('reference_types-1.0.xml')/referenceTypes"/>

    <!-- check references -->
    <xsl:template
            match="*[local-name()='referenceToCommissioner'
        or local-name()='referenceToComplementingProcess'
        or local-name()='referenceToCompleteReviewReport'
        or local-name()='referenceToComplianceSystem'
        or local-name()='referenceToContact'
        or local-name()='referenceToConvertedOriginalDataSetFrom'
        or local-name()='referenceToDataHandlingPrinciples'
        or local-name()='referenceToDataSetFormat'
        or local-name()='referenceToDataSetUseApproval'
        or local-name()='referenceToDataSource'
        or local-name()='referenceToDiagram'
        or local-name()='referenceToEntitiesWithExclusiveAccess'
        or local-name()='referenceToExternalDocumentation'
        or local-name()='referenceToFlowDataSet'
        or local-name()='referenceToFlowPropertyDataSet'
        or local-name()='referenceToIncludedProcesses'
        or local-name()='referenceToIncludedSubMethods'
        or local-name()='referenceToLCAMethodDetails'
        or local-name()='referenceToLCIAMethodDataSet'
        or local-name()='referenceToLogo'
        or local-name()='referenceToNameOfReviewerAndInstitution'
        or local-name()='referenceToOriginalEPD'
        or local-name()='referenceToOwnershipOfDataSet'
        or local-name()='referenceToPersonOrEntityEnteringTheData'
        or local-name()='referenceToPersonOrEntityGeneratingTheDataSet'
        or local-name()='referenceToPrecedingDataSetVersion'
        or local-name()='referenceToPrecedingDataSetVersion'
        or local-name()='referenceToProcess'
        or local-name()='referenceToPublisher'
        or local-name()='referenceToReferenceUnitGroup'
        or local-name()='referenceToRegistrationAuthority'
        or local-name()='referenceToResultingProcess'
        or local-name()='referenceToSource'
        or local-name()='referenceToSupportedImpactAssessmentMethods'
        or local-name()='referenceToTechnicalSpecification'
        or local-name()='referenceToTechnologyFlowDiagrammOrPicture'
        or local-name()='referenceToTechnologyPictogramme'
        or local-name()='referenceToUnchangedRepublication'
        or local-name()='referenceToVendor'
        or local-name()='isA']">

        <xsl:call-template name="checkReference"/>
    </xsl:template>


    <!-- check a reference -->
    <xsl:template name="checkReference">

        <xsl:variable name="elementName" select="name()"/>
        <xsl:variable name="typeFound" select="@type"/>

        <!-- check for correct type attribute  -->
        <xsl:variable name="correctType">
            <xsl:call-template name="determineReferenceType">
                <xsl:with-param name="for" select="local-name()"/>
                <xsl:with-param name="referenceTypes" select="$referenceTypes"/>
            </xsl:call-template>
        </xsl:variable>

        <xsl:if test="@type!=$correctType">
            <xsl:call-template name="validationEvent">
                <xsl:with-param name="fancyMessages" select="$fancyMessages"/>
                <xsl:with-param name="message">Element <xsl:value-of select="name()"/>: reference type "<xsl:value-of select="$correctType"/>" expected, but "<xsl:value-of
                        select="@type"/>" found.</xsl:with-param>
            </xsl:call-template>
        </xsl:if>

        <!-- reference must have either refObjectId or uri, or both -->
        <xsl:if test="not(@refObjectId or @uri)">
            <xsl:call-template name="validationEvent">
                <xsl:with-param name="fancyMessages" select="$fancyMessages"/>
                <xsl:with-param name="message">Element <xsl:value-of select="$elementName"/>: a reference must have either refObjectId or uri attribute, or both.</xsl:with-param>
            </xsl:call-template>
        </xsl:if>

        <!-- check if subReference element is allowed -->
        <xsl:if test="count(common:subReference)>0">
            <xsl:if
                    test="$typeFound!='source data set' or local-name()='referenceToDataSetFormat' or local-name()='referenceToComplianceSystem' or local-name()='referenceToPrecedingDataSetVersion' or local-name()='referenceToTechnologyFlowDiagrammOrPicture' or local-name()='referenceToTechnologyPictogramme' or local-name()='logo'">
                <xsl:call-template name="validationEvent">
                    <xsl:with-param name="fancyMessages" select="$fancyMessages"/>
                    <xsl:with-param name="message">Element <xsl:value-of select="$elementName"/>: a subReference element is not allowed for this reference.</xsl:with-param>
                </xsl:call-template>
            </xsl:if>
        </xsl:if>

    </xsl:template>

    <xsl:template match="source:referenceToDigitalFile">
        <!-- allowed characters are a-z A-Z 0-9 . - _ , ( ) [space] -->
        <xsl:variable name="vUpper" select="'ABCDEFGHIJKLMNOPQRSTUVWXYZ'"/>
        <xsl:variable name="vLower" select="'abcdefghijklmnopqrstuvwxyz'"/>
        <xsl:variable name="vAlpha" select="concat($vUpper, $vLower)"/>
        <xsl:variable name="vDigits" select="'0123456789'"/>
        <xsl:variable name="vOther" select="'.-_(), '"/>
        <xsl:variable name="vAll" select="concat($vAlpha, $vDigits, $vOther)"/>

        <!-- if it refers to an external URL, we're done here, otherwise check the filename part -->
        <xsl:if test="not(starts-with(@uri, 'https://') or starts-with(@uri, 'http://'))">
            <xsl:variable name="filename">
                <xsl:call-template name="substringAfterLast">
                    <!-- we convert any potential backslashes in forward ones -->
                    <xsl:with-param name="string1" select="translate(@uri, '\', '/')"/>
                    <xsl:with-param name="string2" select="'/'"/>
                </xsl:call-template>
            </xsl:variable>

            <xsl:variable name="illegalChars" select="translate($filename,$vAll,'')"/>

            <xsl:if test="string-length($illegalChars) != 0">
                <xsl:call-template name="validationEvent">
                    <xsl:with-param name="fancyMessages" select="$fancyMessages"/>
                    <xsl:with-param name="message">The file name in referenceToDigitalFile '<xsl:value-of select="$filename"/>' contains illegal characters '<xsl:value-of select="$illegalChars"/>'. Allowed characters are a-z A-Z 0-9 . - _ and spaces.</xsl:with-param>
                </xsl:call-template>
            </xsl:if>
        </xsl:if>

    </xsl:template>

</xsl:stylesheet>
